diff --git a/src/pl/plperl/plperl.c b/src/pl/plperl/plperl.c
index 270e9f7..e269c2a 100644
--- a/src/pl/plperl/plperl.c
+++ b/src/pl/plperl/plperl.c
@@ -113,7 +113,7 @@ typedef struct plperl_proc_desc
 
 /**********************************************************************
  * For speedy lookup, we maintain a hash table mapping from
- * function OID + trigger flag + user OID to plperl_proc_desc pointers.
+ * function OID + user OID to plperl_proc_desc pointers.
  * The reason the plperl_proc_desc struct isn't directly part of the hash
  * entry is to simplify recovery from errors during compile_plperl_function.
  *
@@ -127,11 +127,6 @@ typedef struct plperl_proc_desc
 typedef struct plperl_proc_key
 {
 	Oid			proc_id;				/* Function OID */
-	/*
-	 * is_trigger is really a bool, but declare as Oid to ensure this struct
-	 * contains no padding
-	 */
-	Oid			is_trigger;				/* is it a trigger function? */
 	Oid			user_id;				/* User calling the function, or 0 */
 } plperl_proc_key;
 
@@ -1958,9 +1953,20 @@ compile_plperl_function(Oid fn_oid, bool is_trigger)
 	plperl_error_context.arg = NameStr(procStruct->proname);
 	error_context_stack = &plperl_error_context;
 
+	/*
+	 * if we are a trigger, but not called as one, bail out.  otherwise we
+	 * might have the wrong (or no) input/output conversion setup.
+	 * We need to check here because trigger or not we would have the same
+	 * plperl_proc_hash entry
+	 */
+	if(!is_trigger && procStruct->prorettype == TRIGGEROID)
+		ereport(ERROR,
+			(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+			 errmsg("trigger functions can only be called "
+			"as triggers")));
+
 	/* Try to find function in plperl_proc_hash */
 	proc_key.proc_id = fn_oid;
-	proc_key.is_trigger = is_trigger;
 	proc_key.user_id = GetUserId();
 
 	proc_ptr = hash_search(plperl_proc_hash, &proc_key,
@@ -2057,15 +2063,6 @@ compile_plperl_function(Oid fn_oid, bool is_trigger)
 				if (procStruct->prorettype == VOIDOID ||
 					procStruct->prorettype == RECORDOID)
 					 /* okay */ ;
-				else if (procStruct->prorettype == TRIGGEROID)
-				{
-					free(prodesc->proname);
-					free(prodesc);
-					ereport(ERROR,
-							(errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
-							 errmsg("trigger functions can only be called "
-									"as triggers")));
-				}
 				else
 				{
 					free(prodesc->proname);
